'use strict';

/**
 * @class Validation に関するクラス
 * @param {string} PROTOCOL_REGEXP Protocolをチェックするための正規表現リテラル
 * @param {string} IP_REGEXP IPをチェックするための正規表現リテラル
 * @param {string} MASK_REGEXP Maskをチェックするための正規表現リテラル
 * @param {string} CIDR_REGEXP CIDRをチェックするための正規表現リテラル
 * @param {string} POST_REGEXP POSTをチェックするための正規表現リテラル
 * @param {int} VLANID_REGEXP VLANIDをチェックするための正規表現リテラル 1 ~ 4096
 * @param {string} RESULT_REGEXP Resultをチェックするための正規表現リテラル
 */
var Validater = function () {
    const PROTOCOL_REGEXP = /^(icmp|tcp|udp)$/
    const IP_REGEXP = /^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/
    const MASK_REGEXP = /^([1-9]|[1-2][0-9]|30)$/
    const CIDR_REGEXP = /^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\/([1-9]|[1-2][0-9]|30)$/
    const PORT_REGEXP = /^([1-9]|[1-9][0-9]|[1-9][0-9][0-9]|[1-9][0-9][0-9][0-9]|[1-5][0-9][0-9][0-9][0-9]|6[0-4][0-9][0-9][0-9]|65[0-4][0-9][0-9]|655[0-2][0-9]|6553[0-5])$/
    const VLANID_REGEXP = /^[1-9]$|^[1-9][0-9]$|^[1-9][0-9]{2}$|^[1-3][0-9]{3}$|^40[0-8][0-9]$|^409[0-4]$/
    const RESULT_REGEXP = /^(pass|drop)$/
    /**
     * Protocolのバリデーションを行う関数
     * @param {string} value チェックする値
     * @return {boolean} 正規表現に一致すればTrueを返す。
     */
    Validater.prototype.Protocol = function (value) {
        return PROTOCOL_REGEXP.test(value);
    }
    /**
     * IPのバリデーションを行う関数
     * @param {string} value チェックする値
     * @return {boolean} 正規表現に一致すればTrueを返す。
     */
    Validater.prototype.Ip = function (value) {
        return IP_REGEXP.test(value);
    }
    /**
     * Maskのバリデーションを行う関数
     * @param {string} value チェックする値
     * @return {boolean} 正規表現に一致すればTrueを返す。
     */
    Validater.prototype.Mask = function (value) {
        return MASK_REGEXP.test(value);
    }
    /**
     * Cidrのバリデーションを行う関数
     * @param {string} value チェックする値
     * @return {boolean} 正規表現に一致すればTrueを返す。
     */
    Validater.prototype.Cidr = function (value) {
        return CIDR_REGEXP.test(value);
    }
    /**
     * Portのバリデーションを行う関数
     * @param {string} value チェックする値
     * @return {boolean} 正規表現に一致すればTrueを返す。
     */
    Validater.prototype.Port = function (value) {
        return PORT_REGEXP.test(value);
    }
    /**
     * Resultのバリデーションを行う関数
     * @param {string} value チェックする値
     * @return {boolean} 正規表現に一致すればTrueを返す。
     */
    Validater.prototype.Result = function (value) {
        return RESULT_REGEXP.test(value);
    }
    /**
    * VlanIdのバリデーションを行う関数
    * @param {int} value チェックする値
    * @return {boolean} 正規表現に一致すればTrueを返す。
    **/
    Validater.prototype.VlanId = function(value) {
        return VLANID_REGEXP.test(value);
    }
}
/**
 * @class 各項目の基本となる変数と関数を定義するクラス (継承して使われる)
 * @property {string} value そのフォームがもつ文字列
 * @property {string} message validation()時に発生したそのフォームのエラー内容の文字列
 * @property {string} selector oneline試験時のそのフォームのセレクタ(jqueryのセレクタ指定に利用)
 * @property {boolean} isInvalid このフォームにエラーが存在するかどうかのフラグ
 * @property {boolean} isEmpty　このフォームが空であるかのフラグ
 */
var FormItem = function () {
    this.value = "";
    this.validater = new Validater();
    this.message = "";
    this.selector = "";
    this.isInvalid = false;
    this.isEmpty = false;
    this.error_messages = null;
    /**
     * フォームが持つ文字列を消去する関数
     */
    FormItem.prototype.DelValue = function () {
        this.value = null;
    }
    /**
     * フォームのDOMの文字列を消去する関数
     */
    FormItem.prototype.DelDOM = function () {
        $(this.selector).val('');
    }
    /**
     * フォームに値をセットする。(DOMは変更なし)
     * @param {string} new_value 新しい値
     */
    FormItem.prototype.setValue = function (new_value) {
        if(typeof new_value === "undefined"){
            new_value="";
        }
        this.value = new_value.replace(/\s+/g, "");
        if (new_value === "") {
            this.isEmpty = true;
        }
    }
    /**
     * 現在の値を返す関数
     * @return {string} 現在の値
     */
    FormItem.prototype.getValue = function () {
        return this.value;
    }
    /** このフォームのエラーメッセージを返す関数
     * @return {string} エラーメッセージ
     * */
    FormItem.prototype.getMessage = function () {
        return this.message;
    }
    /**
     * このフォームのエラーメッセージを消去する関数
     */
    FormItem.prototype.DelMessage = function () {
        this.message = "";
    }
    /**
     * DOMから値を読み込む関数
     */
    FormItem.prototype.setFromDOM = function () {
        this.value = $(this.selector).val().replace(/\s+/g, "");
        if (this.value === "") {
            this.isEmpty = true;
        }
    }
    /**
     * 設定したインターフェイスのipと重複していないかを確認する関数
     * @param {string} checked_ip 試験に利用するip
     * @param {string} name 設定しているフォームの名前["Source","Destination"]
     * @return {boolean} 重複していればTrueを返す。
     */
    FormItem.prototype.CheckDuplicate = function (checked_ip, name) {
        if (needle === null) { console.error("Can't get IPs of NeedleWork"); }
        var verify = "";
        // Eth0との重複チェック
        if (needle.eth0 !== null && needle.eth0 !== "") {
            if (needle.eth0 === checked_ip) {
                verify += ERROR_MESSAGES["CheckDuplicate"].ETH0 + name + ERROR_MESSAGES["CheckDuplicate"].Remaining + "<br>"
            }
        }
        // Eth1との重複チェック
        if (needle.eth1 !== null && needle.eth1 !== "") {
            if (needle.eth1 === checked_ip) {
                verify += ERROR_MESSAGES["CheckDuplicate"].ETH1 + name + ERROR_MESSAGES["CheckDuplicate"].Remaining + "<br>"
            }
        }
        // Eth2との重複チェック
        if (needle.eth2 !== null && needle.eth2 !== "") {
            if (needle.eth2 === checked_ip) {
                verify += ERROR_MESSAGES["CheckDuplicate"].ETH2 + name + ERROR_MESSAGES["CheckDuplicate"].Remaining + "<br>"
            }
        }
        // Mgt0との重複チェック
        if (needle.mgt0 !== null && needle.mgt0 !== "") {
            if (needle.mgt0 === checked_ip) {
                verify += ERROR_MESSAGES["CheckDuplicate"].MGT0 + name + ERROR_MESSAGES["CheckDuplicate"].Remaining + "<br>"
            }
        }
        // Vlanとの重複チェック
        for (let key in needle.vlan) {
            if (needle.vlan[key] === null) { continue; }
            if (needle.vlan[key].length == 0) { continue; }
            for (let i = 0; i < needle.vlan[key].length; i++) {
                if (needle.vlan[key][i].Ip.split('/')[0] === checked_ip) {
                    let vmsg = ERROR_MESSAGES["CheckDuplicate"].VLAN.replace('%tag%', needle.vlan[key][i].Id);
                    verify += vmsg + name + ERROR_MESSAGES["CheckDuplicate"].Remaining + "<br>";
                }
            }
        }

        this.message = verify;
        if (verify !== "") {
            this.isInvalid = true;
            return true;
        }
        return false;
    }
}

/**
 * @class Protocolに関するクラス
 * @augments FormItem
 * @property {boolean} isICMP protocolがICMPであればtrue
 * @property {function} Read_ErrorMessage 別ファイルで定義されたエラーメッセージを読み込む
 */
var Protocol = function () {
    this.isICMP = false;
    this.isTCP = false;
    this.isUDP = false;
    this.selector = "#one-protocol";
    /**
     * バリデーションを行う関数
     */
    Protocol.prototype.Validate = function () {
        if (!this.validater.Protocol(this.value)) {
            this.message = ERROR_MESSAGES["Protocol"].Invalid + "<br>";
            this.isInvalid = true;
        } else {
            this.message = "";
            this.isInvalid = false;
        }
    }
    /**
     * DOMから値を読み込む関数(override)
     */
    Protocol.prototype.setFromDOM = function () {
        //console.info("*Reading value from Dom...");
        switch ($(this.selector).val().replace(/\s+/g, "")) {
            case '2':
                this.value = 'icmp'
                this.isICMP = true;
                break;
            case '3':
                this.value = 'tcp'
                this.isTCP = true;
                break;
            case '4':
                this.value = 'udp'
                this.isUDP = true;
                break;
            default:
                break;
        }
        if (this.value === "") {
            this.isEmpty = true;
        }
        //console.info("Done");
    }
    /**
     * プロトコルを設定する関数
     * @param {string} value protocol名(ICMP,TCP,UDP)
     */
    Protocol.prototype.setValue = function (new_value) {
        this.value = new_value.replace(/\s+/g, "");
        this.isICMP = false;
        this.isTCP = false;
        this.isUDP = false;
        if (this.value === "") {
            this.isEmpty = true;
        } else if (this.value === "icmp") {
            this.isICMP = true;
        } else if (this.value === 'tcp') {
            this.isTCP = true;
        } else if (this.value === 'udp') {
            this.isUDP = true;
        }
    }
}
Protocol.prototype = new FormItem();

/**
 * @class Source FW IPに関するクラス
 * @augments FormItem
*/
var SourceFWIP = function () {
    this.selector = "#one-srcfw";
    /**
     * バリデーションを行う関数
     */
    SourceFWIP.prototype.Validate = function () {
        if (this.CheckDuplicate(this.value, "SourceFW")) {
            this.isInvalid = true;
            return;
        }
        if (!this.validater.Ip(this.value)) {
            this.message = ERROR_MESSAGES["SourceFWIP"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
}
SourceFWIP.prototype = new FormItem();

/**
 * @class Destination FW IP に関するクラス
 * @augments FormItem
 */
var DestinationFWIP = function () {
    this.selector = "#one-dstfw";
    /**
     * バリデーションを行う関数
     */
    DestinationFWIP.prototype.Validate = function () {
        if (this.CheckDuplicate(this.value, "DestinationFW")) {
            this.isInvalid = true;
            return;
        }
        if (!this.validater.Ip(this.value)) {
            this.message = ERROR_MESSAGES["DestinationFWIP"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
}
DestinationFWIP.prototype = new FormItem();

/**
 * @class Source IPに関するクラス
 * @augments FormItem
 */
var SourceIP = function () {
    this.selector = "#one-srcip";
    /**
     * Scenario試験でのバリデーションを行う関数
     */
    SourceIP.prototype.ValidateScenario = function () {
        if (this.CheckDuplicate(this.value, "Source")) {
            this.isInvalid = true;
            return;
        }
        if (!this.validater.Ip(this.value)) {
            this.message = ERROR_MESSAGES["SourceIP"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * Oneline試験でのバリデーションを行う関数
     */
    SourceIP.prototype.ValidateOneLine = function () {
        if (this.CheckDuplicate(this.value, "Source")) {
            this.isInvalid = true;
            return;
        }
        if (!this.validater.Cidr(this.value)) {
            this.message = ERROR_MESSAGES["SourceIP"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * DOMから値を読み込む関数
     */
    SourceIP.prototype.setFromDOM = function () {
        //console.info("*Reading value from Dom...");
        var value = $(this.selector).val().replace(/\s+/g, "");
        this.value = value.split("/")[0];//ipだけを抜き出す。
        if (this.value === "") {
            this.isEmpty = true;
        }
        //console.info("Done")
    }

}
SourceIP.prototype = new FormItem();

/**
 * @class Source Maskに関するクラス
 * @augments FormItem
 */
var SourceMask = function () {
    this.selector = "#one-srcip";
    /**
     * バリデーションを行う関数
     */
    SourceMask.prototype.Validate = function () {
        if (!this.validater.Mask(this.value)) {
            this.message = ERROR_MESSAGES["SourceMask"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * DOMから値を読み込む関数
     */
    SourceMask.prototype.setFromDOM = function () {
        //console.info("*Reading value from Dom...");
        var value = $(this.selector).val().replace(/\s+/g, "");
        this.value = value.split("/")[1];//prefixだけを抜き出す。
        if (this.value === "") {
            this.isEmpty = true;
        }
        //console.info("Done")
    }
}
SourceMask.prototype = new FormItem();

/**
 * @class Source Portに関するクラス
 * @augments FormItem
 */
var SourcePort = function () {
    this.selector = "#one-srcport";
    /**
     * ICMPでのバリデーションを行う関数
     */
    SourcePort.prototype.ValidateICMP = function () {
        if (!this.isEmpty) {
            this.message = ERROR_MESSAGES["SourcePort"].ICMP + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * TCP/UDPでのバリデーションを行う関数
     */
    SourcePort.prototype.ValidateTCPUDP = function () {
        if (!this.isEmpty) {
            if (!this.validater.Port(this.value)) {
                this.message = ERROR_MESSAGES["SourcePort"].Invalid + "<br>";
                this.isInvalid = true;
            }
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
}
SourcePort.prototype = new FormItem();

/**
 * @class Destination IPに関するクラス
 * @augments FormItem
 */
var DestinationIP = function () {
    this.selector = "#one-dstip";
    /**
     * Scenario試験でのバリデーションを行う関数
     */
     DestinationIP.prototype.ValidateScenario = function () {
        if (this.CheckDuplicate(this.value, "Destination")) {
            this.isInvalid = true;
            return;
        }
        if (!this.validater.Ip(this.value)) {
            this.message = ERROR_MESSAGES["DestinationIP"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * Oneline試験でのバリデーションを行う関数
     */
    DestinationIP.prototype.ValidateOneLine = function () {
        if (this.CheckDuplicate(this.value, "Destination")) {
            this.isInvalid = true;
            return;
        }
        if (!this.validater.Cidr(this.value)) {
            this.message = ERROR_MESSAGES["DestinationIP"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * DOMから値を読み込む関数
     */
    DestinationIP.prototype.setFromDOM = function () {
        //console.info("*Reading value from Dom...");
        var value = $(this.selector).val().replace(/\s+/g, "");
        this.value = value.split("/")[0];//ipだけを抜き出す。
        if (this.value === "") {
            this.isEmpty = true;
        }
        //console.info("Done")
    }
}
DestinationIP.prototype = new FormItem();

/**
 * @class Destination Maskに関するクラス
 * @augments FormItem
 */
var DestinationMask = function () {
    this.selector = "#one-dstip";
    /**
     * バリデーションを行う関数
     */
    DestinationMask.prototype.Validate = function () {
        if (!this.validater.Mask(this.value)) {
            this.message = ERROR_MESSAGES["DestinationMask"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * DOMから値を読み込む関数
     */
    DestinationMask.prototype.setFromDOM = function () {
        //console.info("*Reading value from Dom...");
        var value = $(this.selector).val().replace(/\s+/g, "");
        this.value = value.split("/")[1];//prefixだけを抜き出す。
        if (this.value === "") {
            this.isEmpty = true;
        }
        //console.info("Done.")
    }
}
DestinationMask.prototype = new FormItem();

/**
 * @class Destination Portに関するクラス
 * @augments FormItem
 */
var DestinationPort = function () {
    this.selector = "#one-dstport";
    /**
     * ICMPでのバリデーションを行う関数
     */
    DestinationPort.prototype.ValidateICMP = function () {
        if (!this.isEmpty) {
            this.message = ERROR_MESSAGES["DestinationPort"].ICMP + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * TCP/UDPでのバリデーションを行う関数
     */
    DestinationPort.prototype.ValidateTCPUDP = function () {
        if (this.isEmpty) {
            this.message = ERROR_MESSAGES["DestinationPort"].Require + "<br>";
            this.isInvalid = true;
            return;
        } else if (!this.validater.Port(this.value)) {
            this.message = ERROR_MESSAGES["DestinationPort"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
}
DestinationPort.prototype = new FormItem();

/**
 * @class Destination Nat IPに関するクラス
 * @augments FormItem
 */
var DestinationNatIP = function () {
    this.selector = "#one-nat-dstip";
    /**
     * バリデーションを行う関数
     */
    DestinationNatIP.prototype.Validate = function () {
        if (!this.isEmpty) {//正規表現を避けるため
            if (this.CheckDuplicate(this.value, "DestinationNat")) {
                this.isInvalid = true;
                return;
            }
            if (!this.validater.Ip(this.value)) {
                this.message = ERROR_MESSAGES["DestinationNatIP"].Invalid + "<br>";
                this.isInvalid = true;
            }
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * DOMから値を読み込む関数
     */
    DestinationNatIP.prototype.setFromDOM = function () {
        //console.info("*Reading value from Dom...");
        this.value = $(this.selector).val().replace(/\s+/g, "");
        if (this.value === "") {
            this.isEmpty = true;
            return;
        }
        this.isExist_NatIp = true;
        //console.info("Done")
    }
}
DestinationNatIP.prototype = new FormItem();

/**
 * @class Destination Nat Port に関するクラス
 * @augments FormItem
 */
var DestinationNatPort = function () {
    this.selector = "#one-nat-dstport";
    /**
     * ICMPでのバリデーションを行う関数
     */
    DestinationNatPort.prototype.ValidateICMP = function () {
        if (!this.isEmpty) {
            this.message = ERROR_MESSAGES["DestinationNatPort"].ICMP + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * TCP/UDPでのバリデーションを行う関数
     */
    DestinationNatPort.prototype.ValidateTCPUDP = function () {
        if (!this.isEmpty) {
            if (!this.validater.Port(this.value)) {
                this.message = ERROR_MESSAGES["DestinationNatPort"].Invalid + "<br>";
                this.isInvalid = true;
            }
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
    /**
     * TCP/UDPでNat IPが存在する場合のバリデーションを行う関数
     */
    DestinationNatPort.prototype.ValidateTCPUDPNat = function () {
        //console.log("nat port", this.value)
        if (this.value !== "") {
            this.message = ERROR_MESSAGES["DestinationNatPort"].NoIP + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }
}
DestinationNatPort.prototype = new FormItem();

/**
 * @class Except Resultに関するクラス
 * @augments FormItem
 */
var ExpectedResult = function () {
    /**
     * Except Resultのバリデーションを行う関数
     */
    ExpectedResult.prototype.ValidateScenario = function () {
        if (this.isEmpty) {
            this.message = ERROR_MESSAGES["ExpectedResult"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        } else if (!this.validater.Result(this.value)) {
            this.message = ERROR_MESSAGES["ExpectedResult"].Invalid + "<br>";
            this.isInvalid = true;
            return;
        }
        this.message = "";
        this.isInvalid = false;
    }

}
ExpectedResult.prototype = new FormItem();
